@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -54,19 +54,18 @@
 	                    StringValuePtr (arg1), StringValuePtr (arg2)); \
 	METHOD_HANDLER_FOOTER
 
-#define METHOD_ADD_HANDLER_UINT(name, arg1) \
-	METHOD_HANDLER_HEADER \
-\
-	res = xmmsc_##name (xmms->real, check_uint32 (arg1)); \
-\
-	METHOD_HANDLER_FOOTER
-
 #define METHOD_ADD_HANDLER_INT(name, arg1) \
 	METHOD_HANDLER_HEADER \
 \
 	res = xmmsc_##name (xmms->real, check_int32 (arg1)); \
 	METHOD_HANDLER_FOOTER
 
+#define METHOD_ADD_HANDLER_INT_INT(name, arg1, arg2) \
+	METHOD_HANDLER_HEADER \
+\
+	res = xmmsc_##name (xmms->real, check_int32 (arg1), check_int32 (arg2)); \
+	METHOD_HANDLER_FOOTER
+
 #define METHOD_ADD_HANDLER_BIN(name, arg1) \
 	METHOD_HANDLER_HEADER \
 \
@@ -77,6 +76,8 @@
 	                    RSTRING_LEN (arg1)); \
 	METHOD_HANDLER_FOOTER
 
+#define rb_check_hash_type(x) rb_check_convert_type(x, T_HASH, "Hash", "to_hash")
+
 static VALUE cPlaylist;
 static VALUE eClientError, eDisconnectedError;
 static ID id_lt, id_gt;
@@ -345,7 +346,7 @@ c_io_in_handle (VALUE self)
  *  xc.io_out_handle -> nil
  *
  * Sends one outgoing (to server) clientlib command. You should check
- * Xmms::Client#io_want_out before calling this method.
+ * #io_want_out before calling this method.
  */
 static VALUE
 c_io_out_handle (VALUE self)
@@ -547,7 +548,13 @@ c_broadcast_config_value_changed (VALUE self)
 static VALUE
 c_playback_seek_ms (VALUE self, VALUE ms)
 {
-	METHOD_ADD_HANDLER_UINT (playback_seek_ms_abs, ms);
+	METHOD_HANDLER_HEADER
+
+	res = xmmsc_playback_seek_ms (xmms->real,
+	                              check_int32 (ms),
+	                              XMMS_PLAYBACK_SEEK_SET);
+
+	METHOD_HANDLER_FOOTER
 }
 
 /*
@@ -559,7 +566,13 @@ c_playback_seek_ms (VALUE self, VALUE ms)
 static VALUE
 c_playback_seek_ms_rel (VALUE self, VALUE ms)
 {
-	METHOD_ADD_HANDLER_INT (playback_seek_ms_rel, ms);
+	METHOD_HANDLER_HEADER
+
+	res = xmmsc_playback_seek_ms (xmms->real,
+	                              check_int32 (ms),
+	                              XMMS_PLAYBACK_SEEK_CUR);
+
+	METHOD_HANDLER_FOOTER
 }
 
 /*
@@ -571,7 +584,13 @@ c_playback_seek_ms_rel (VALUE self, VALUE ms)
 static VALUE
 c_playback_seek_samples (VALUE self, VALUE samples)
 {
-	METHOD_ADD_HANDLER_UINT (playback_seek_samples_abs, samples);
+	METHOD_HANDLER_HEADER
+
+	res = xmmsc_playback_seek_samples (xmms->real,
+	                                   check_int32 (samples),
+	                                   XMMS_PLAYBACK_SEEK_SET);
+
+	METHOD_HANDLER_FOOTER
 }
 
  /*
@@ -583,7 +602,13 @@ c_playback_seek_samples (VALUE self, VALUE samples)
 static VALUE
 c_playback_seek_samples_rel (VALUE self, VALUE samples)
 {
-	METHOD_ADD_HANDLER_INT (playback_seek_samples_rel, samples);
+	METHOD_HANDLER_HEADER
+
+	res = xmmsc_playback_seek_samples (xmms->real,
+	                                   check_int32 (samples),
+	                                   XMMS_PLAYBACK_SEEK_CUR);
+
+	METHOD_HANDLER_FOOTER
 }
 
 /*
@@ -607,7 +632,7 @@ c_playback_volume_set (VALUE self, VALUE channel, VALUE volume)
 
 	res = xmmsc_playback_volume_set (xmms->real,
 	                                 rb_id2name (SYM2ID (channel)),
-	                                 NUM2UINT (volume));
+	                                 NUM2INT (volume));
 
 	return TO_XMMS_CLIENT_RESULT (self, res);
 }
@@ -692,7 +717,20 @@ c_broadcast_playlist_loaded (VALUE self)
 static VALUE
 c_broadcast_medialib_entry_changed (VALUE self)
 {
-	METHOD_ADD_HANDLER (broadcast_medialib_entry_changed);
+	RB_XMMS_DEPRECATED (broadcast_medialib_entry_changed, broadcast_medialib_entry_updated);
+	METHOD_ADD_HANDLER (broadcast_medialib_entry_updated);
+}
+
+/*
+ * call-seq:
+ *  xc.broadcast_medialib_entry_changed -> result
+ *
+ * Retrieves the id of a changed medialib entry as a broadcast.
+ */
+static VALUE
+c_broadcast_medialib_entry_updated (VALUE self)
+{
+	METHOD_ADD_HANDLER (broadcast_medialib_entry_updated);
 }
 
 /*
@@ -707,6 +745,18 @@ c_broadcast_medialib_entry_added (VALUE self)
 	METHOD_ADD_HANDLER (broadcast_medialib_entry_added);
 }
 
+/*
+ * call-seq:
+ *  xc.broadcast_medialib_entry_removed -> result
+ *
+ * Retrieves the id of an removed medialib entry as a broadcast.
+ */
+static VALUE
+c_broadcast_medialib_entry_removed (VALUE self)
+{
+	METHOD_ADD_HANDLER (broadcast_medialib_entry_removed);
+}
+
 /*
  * call-seq:
  *  xc.playlist_set_next(pos) -> result
@@ -716,7 +766,7 @@ c_broadcast_medialib_entry_added (VALUE self)
 static VALUE
 c_playlist_set_next (VALUE self, VALUE pos)
 {
-	METHOD_ADD_HANDLER_UINT (playlist_set_next, pos);
+	METHOD_ADD_HANDLER_INT (playlist_set_next, pos);
 }
 
 /*
@@ -735,7 +785,7 @@ c_playlist_set_next_rel (VALUE self, VALUE pos)
 /* call-seq:
  * xc.playlist([name]) -> p
  *
- * Shortcut for Xmms::Playlist#new(xc, [name]). Creates a new Xmms::Playlist
+ * Shortcut for Xmms::Playlist.new. Creates a new Xmms::Playlist
  * object tied to the current Xmms::Client instance. _name_ is is the name of
  * the playlist and the active playlist will be used if it is not specified.
  * Raises PlaylistError if the playlist name is invalid.
@@ -783,7 +833,7 @@ c_medialib_get_id (VALUE self, VALUE url)
 static VALUE
 c_medialib_get_info (VALUE self, VALUE id)
 {
-	METHOD_ADD_HANDLER_UINT (medialib_get_info, id);
+	METHOD_ADD_HANDLER_INT (medialib_get_info, id);
 }
 
 /*
@@ -793,7 +843,7 @@ c_medialib_get_info (VALUE self, VALUE id)
  * Write info to the medialib at _id_. _source_ is an optional argument that
  * describes where to write the mediainfo. If _source_ is omitted, the
  * mediainfo is written to "client/<yourclient>" where <yourclient> is the
- * name you specified in _Xmms::Client.new(name)_.
+ * name you specified in ::new.
  */
 static VALUE
 c_medialib_entry_property_set (int argc, VALUE *argv, VALUE self)
@@ -803,8 +853,7 @@ c_medialib_entry_property_set (int argc, VALUE *argv, VALUE self)
 	xmmsc_result_t *res;
 	const char *ckey;
 	bool is_str = false;
-	uint32_t id;
-	int32_t ivalue;
+	int32_t id, ivalue;
 
 	Data_Get_Struct (self, RbXmmsClient, xmms);
 
@@ -812,7 +861,7 @@ c_medialib_entry_property_set (int argc, VALUE *argv, VALUE self)
 
 	rb_scan_args (argc, argv, "31", &tmp, &key, &value, &src);
 
-	id = check_uint32 (tmp);
+	id = check_int32 (tmp);
 	Check_Type (key, T_SYMBOL);
 
 	if (!NIL_P (rb_check_string_type (value)))
@@ -851,8 +900,7 @@ c_medialib_entry_property_set (int argc, VALUE *argv, VALUE self)
  * Remove a custom field in the medialib associated with the entry _id_.
  * _source_ is an optional argument that describes where to write the
  * mediainfo. If _source_ is omitted, "client/<yourclient>" is used,
- * where <yourclient> is the name you specified in
- * _Xmms::Client.new(name)_.
+ * where <yourclient> is the name you specified in ::new.
  */
 static VALUE
 c_medialib_entry_property_remove (int argc, VALUE *argv, VALUE self)
@@ -861,7 +909,7 @@ c_medialib_entry_property_remove (int argc, VALUE *argv, VALUE self)
 	RbXmmsClient *xmms = NULL;
 	xmmsc_result_t *res;
 	const char *ckey;
-	uint32_t id;
+	int32_t id;
 
 	Data_Get_Struct (self, RbXmmsClient, xmms);
 
@@ -869,7 +917,7 @@ c_medialib_entry_property_remove (int argc, VALUE *argv, VALUE self)
 
 	rb_scan_args (argc, argv, "21", &tmp, &key, &src);
 
-	id = check_uint32 (tmp);
+	id = check_int32 (tmp);
 	Check_Type (key, T_SYMBOL);
 
 	ckey = rb_id2name (SYM2ID (key));
@@ -897,7 +945,7 @@ c_medialib_entry_move (VALUE self, VALUE id, VALUE url)
 {
 	METHOD_HANDLER_HEADER
 
-	res = xmmsc_medialib_move_entry (xmms->real, check_uint32 (id),
+	res = xmmsc_medialib_move_entry (xmms->real, check_int32 (id),
 	                                 StringValuePtr (url));
 
 	METHOD_HANDLER_FOOTER
@@ -912,7 +960,7 @@ c_medialib_entry_move (VALUE self, VALUE id, VALUE url)
 static VALUE
 c_medialib_entry_remove (VALUE self, VALUE id)
 {
-	METHOD_ADD_HANDLER_UINT (medialib_remove_entry, id)
+	METHOD_ADD_HANDLER_INT (medialib_remove_entry, id)
 }
 
 /*
@@ -950,7 +998,19 @@ c_playlist_current_active (VALUE self)
 static VALUE
 c_medialib_path_import (VALUE self, VALUE path)
 {
-	METHOD_ADD_HANDLER_STR (medialib_path_import, path);
+	METHOD_ADD_HANDLER_STR (medialib_import_path, path);
+}
+
+/*
+ * call-seq:
+ *  xc.medialib_path_import(path) -> result
+ *
+ * Recursively imports all media files under the url encoded _path_ to the medialib.
+ */
+static VALUE
+c_medialib_path_import_encoded (VALUE self, VALUE path)
+{
+	METHOD_ADD_HANDLER_STR (medialib_import_path_encoded, path);
 }
 
 /*
@@ -962,7 +1022,7 @@ c_medialib_path_import (VALUE self, VALUE path)
 static VALUE
 c_medialib_rehash (VALUE self, VALUE id)
 {
-	METHOD_ADD_HANDLER_UINT (medialib_rehash, id);
+	METHOD_ADD_HANDLER_INT (medialib_rehash, id);
 }
 
 /*
@@ -1017,7 +1077,7 @@ c_plugin_list (int argc, VALUE *argv, VALUE self)
 	if (NIL_P (type))
 		type = INT2FIX (XMMS_PLUGIN_TYPE_ALL);
 
-	METHOD_ADD_HANDLER_UINT (plugin_list, type);
+	METHOD_ADD_HANDLER_INT (main_list_plugins, type);
 }
 
 /*
@@ -1160,7 +1220,7 @@ c_coll_get (int argc, VALUE *argv, VALUE self)
  *
  * Retrieves an array of the names of all the collections stored in the
  * medialib under the namespace _ns_. If _ns_ is not specified, it defaults to
- * ALL.
+ * Xmms::Collection::NS_ALL.
  */
 static VALUE
 c_coll_list (int argc, VALUE *argv, VALUE self)
@@ -1236,7 +1296,7 @@ c_coll_find (VALUE self, VALUE id, VALUE ns)
 {
 	METHOD_HANDLER_HEADER
 
-	res = xmmsc_coll_find (xmms->real, check_uint32 (id),
+	res = xmmsc_coll_find (xmms->real, check_int32 (id),
 	                       StringValuePtr (ns));
 
 	METHOD_HANDLER_FOOTER
@@ -1271,6 +1331,91 @@ c_coll_rename (int argc, VALUE *argv, VALUE self)
 	METHOD_HANDLER_FOOTER
 }
 
+static int
+parse_fetch_spec_foreach (VALUE key, VALUE value, VALUE udata)
+{
+	xmmsv_t *spec = (xmmsv_t *) udata;
+	xmmsv_t *elem;
+	int i;
+
+	if (NIL_P (rb_check_string_type (key))) {
+		rb_raise (rb_eArgError, "Key must be string");
+	}
+
+	if (!NIL_P (rb_check_string_type (value))) {
+		xmmsv_dict_set_string (spec,
+		                       StringValuePtr (key),
+		                       StringValuePtr (value));
+	} else if (!NIL_P (rb_check_hash_type (value))) {
+		elem = xmmsv_new_dict ();
+		xmmsv_dict_set (spec, StringValuePtr (key), elem);
+		xmmsv_unref (elem);
+		rb_hash_foreach (value, parse_fetch_spec_foreach, (VALUE) elem);
+	} else if (!NIL_P (rb_check_array_type (value))) {
+		elem = xmmsv_new_list ();
+		xmmsv_dict_set (spec, StringValuePtr (key), elem);
+		xmmsv_unref (elem);
+
+		for (i = 0; i < RARRAY_LEN (value); i++) {
+			VALUE entry = RARRAY_PTR (value)[i];
+			if (NIL_P (rb_check_string_type (entry)))
+				rb_raise (rb_eArgError, "Value must be hash, string, or array of strings.");
+			xmmsv_list_append_string (elem, StringValuePtr (entry));
+		}
+	} else {
+		rb_raise (rb_eArgError, "Value must be hash, string, or array of strings.");
+	}
+
+	return 0;
+}
+
+static VALUE
+c_coll_query_fragile (VALUE args)
+{
+	VALUE xmms, coll, spec, cspec;
+
+	Check_Type(args, T_ARRAY);
+	rb_scan_args(RARRAY_LEN (args), RARRAY_PTR (args), "4",
+	             &xmms, &coll, &spec, &cspec);
+
+	rb_hash_foreach (spec, parse_fetch_spec_foreach, cspec);
+
+	return (VALUE) xmmsc_coll_query (((RbXmmsClient *) xmms)->real,
+	                                 FROM_XMMS_CLIENT_COLLECTION (coll),
+	                                 (xmmsv_t *) cspec);
+}
+
+static VALUE
+c_coll_query_cleanup (VALUE args)
+{
+	xmmsv_unref ((xmmsv_t *) args);
+	return Qnil;
+}
+
+/*
+ * call-seq:
+ * xc.coll_query(coll, spec) -> result
+ *
+ */
+static VALUE
+c_coll_query (int argc, VALUE *argv, VALUE self)
+{
+	VALUE coll, spec;
+	xmmsv_t *cspec;
+	METHOD_HANDLER_HEADER
+
+	rb_scan_args (argc, argv, "2", &coll, &spec);
+
+	cspec = xmmsv_new_dict ();
+
+	VALUE args = rb_ary_new3(4, xmms, coll, spec, (VALUE) cspec);
+
+	res = (xmmsc_result_t *) rb_ensure (c_coll_query_fragile, args,
+	                                    c_coll_query_cleanup, (VALUE) cspec);
+
+	METHOD_HANDLER_FOOTER
+}
+
 /*
  * call-seq:
  * xc.coll_query_ids(coll, [order], [start], [len]) -> result
@@ -1295,10 +1440,11 @@ c_coll_query_ids (int argc, VALUE *argv, VALUE self)
 	res = xmmsc_coll_query_ids (xmms->real,
 	                            FROM_XMMS_CLIENT_COLLECTION (coll),
 	                            corder,
-	                            NIL_P (start) ? 0 : NUM2UINT (start),
-	                            NIL_P (start) ? 0 : NUM2UINT (len));
+	                            NIL_P (start) ? 0 : NUM2INT (start),
+	                            NIL_P (start) ? 0 : NUM2INT (len));
 
-	xmmsv_unref (corder);
+	if (corder)
+		xmmsv_unref (corder);
 
 	METHOD_HANDLER_FOOTER
 }
@@ -1335,8 +1481,8 @@ c_coll_query_info (int argc, VALUE *argv, VALUE self)
 	res = xmmsc_coll_query_infos (xmms->real,
 	                            FROM_XMMS_CLIENT_COLLECTION (coll),
 	                            corder,
-	                            NIL_P (start) ? 0 : NUM2UINT (start),
-	                            NIL_P (start) ? 0 : NUM2UINT (len),
+	                            NIL_P (start) ? 0 : NUM2INT (start),
+	                            NIL_P (start) ? 0 : NUM2INT (len),
 	                            cfetch,
 	                            cgroup);
 	xmmsv_unref (cfetch);
@@ -1376,30 +1522,11 @@ c_broadcast_coll_changed (VALUE self)
 	METHOD_ADD_HANDLER (broadcast_collection_changed)
 }
 
-VALUE
-check_uint32 (VALUE arg)
-{
-	VALUE uint32_max = UINT2NUM (4294967295UL);
-	VALUE uint32_min = INT2NUM (0);
-
-	if (!rb_obj_is_kind_of (arg, rb_cInteger))
-		rb_raise (rb_eTypeError,
-		          "wrong argument type %s (expected Integer)",
-		          rb_obj_classname (arg));
-
-	if (rb_funcall2 (arg, id_lt, 1, &uint32_min) ||
-	    rb_funcall2 (arg, id_gt, 1, &uint32_max))
-		rb_raise (rb_eTypeError,
-		          "wrong argument type (expected 32 bit unsigned int)");
-
-	return NUM2UINT (arg);
-}
-
 VALUE
 check_int32 (VALUE arg)
 {
-	VALUE int32_max = INT2NUM (2147483647);
-	VALUE int32_min = INT2NUM (-2147483647);
+	VALUE int32_max = INT2NUM (INT_MAX);
+	VALUE int32_min = INT2NUM (INT_MIN);
 
 	if (!rb_obj_is_kind_of (arg, rb_cInteger))
 		rb_raise (rb_eTypeError,
@@ -1475,6 +1602,10 @@ parse_string_array2 (VALUE value)
 void
 Init_Client (VALUE mXmms)
 {
+#if RDOC_CAN_PARSE_DOCUMENTATION
+	mXmms= rb_define_module("Xmms");
+#endif
+
 	VALUE c;
 
 	c = rb_define_class_under (mXmms, "Client", rb_cObject);
@@ -1531,8 +1662,12 @@ Init_Client (VALUE mXmms)
 	                  c_broadcast_playlist_loaded, 0);
 	rb_define_method (c, "broadcast_medialib_entry_changed",
 	                  c_broadcast_medialib_entry_changed, 0);
+	rb_define_method (c, "broadcast_medialib_entry_updated",
+	                  c_broadcast_medialib_entry_updated, 0);
 	rb_define_method (c, "broadcast_medialib_entry_added",
 	                  c_broadcast_medialib_entry_added, 0);
+	rb_define_method (c, "broadcast_medialib_entry_removed",
+	                  c_broadcast_medialib_entry_removed, 0);
 
 	rb_define_method (c, "playlist", c_playlist, -1);
 	rb_define_method (c, "playlist_list", c_playlist_list, 0);
@@ -1547,6 +1682,7 @@ Init_Client (VALUE mXmms)
 	rb_define_method (c, "coll_remove", c_coll_remove, -1);
 	rb_define_method (c, "coll_find", c_coll_find, 2);
 	rb_define_method (c, "coll_rename", c_coll_rename, -1);
+	rb_define_method (c, "coll_query", c_coll_query, -1);
 	rb_define_method (c, "coll_query_ids", c_coll_query_ids, -1);
 	rb_define_method (c, "coll_query_info", c_coll_query_info, -1);
 	rb_define_method (c, "coll_idlist_from_playlist_file",
@@ -1563,6 +1699,7 @@ Init_Client (VALUE mXmms)
 	rb_define_method (c, "medialib_entry_remove", c_medialib_entry_remove, 1);
 	rb_define_method (c, "medialib_entry_move", c_medialib_entry_move, 2);
 	rb_define_method (c, "medialib_path_import", c_medialib_path_import, 1);
+	rb_define_method (c, "medialib_path_import_encoded", c_medialib_path_import_encoded, 1);
 	rb_define_method (c, "medialib_rehash", c_medialib_rehash, 1);
 
 	rb_define_method (c, "xform_media_browse", c_xform_media_browse, 1);
