@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -17,6 +17,9 @@
 #ifndef __MATCHING_BROWSE_H_
 #define __MATCHING_BROWSE_H_
 
+#include <glib.h>
+#include <xmmsclient/xmmsclient.h>
+
 typedef struct browse_entry_St browse_entry_t;
 
 void browse_entry_get (browse_entry_t *data, const gchar **url, gboolean *is_directory);
