@@ -1,4 +1,4 @@
-#include "xmmsc/xmmsc_sockets.h"
+#include <xmmsc/xmmsc_sockets.h>
 
 int xmms_sockets_initialize () {
 	WSADATA wsaData;
