@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -18,14 +18,14 @@
 #include <tremor/ivorbiscodec.h>
 #include <tremor/ivorbisfile.h>
 
-#include "xmms/xmms_xformplugin.h"
+#include <xmms/xmms_xformplugin.h>
 
 #include "../vorbis_common/common.c"
 
-XMMS_XFORM_PLUGIN ("tremor",
-                   "Vorbis Decoder (tremor)", XMMS_VERSION,
-                   "Xiph's Ogg/Vorbis decoder (integer-only version)",
-                   xmms_vorbis_plugin_setup);
+XMMS_XFORM_PLUGIN_DEFINE ("tremor",
+                          "Vorbis Decoder (tremor)", XMMS_VERSION,
+                          "Xiph's Ogg/Vorbis decoder (integer-only version)",
+                          xmms_vorbis_plugin_setup);
 
 static void
 xmms_vorbis_set_duration (xmms_xform_t *xform, guint dur)
