@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -16,21 +16,21 @@
 
 /* Have thread safe localtime when POSIX localtime_r isn't available */
 
-#include "xmmspriv/xmms_localtime.h"
+#include <xmmspriv/xmms_localtime.h>
 
-static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+G_LOCK_DEFINE_STATIC (mutex);
 
 gboolean
 xmms_localtime (const time_t *tt, struct tm *res)
 {
 	struct tm *ret = NULL;
 
-	g_static_mutex_lock (&mutex);
+	G_LOCK (mutex);
 	if ((ret = localtime (tt))) {
 		memcpy (res, ret, sizeof (struct tm));
 		ret = res;
 	}
-	g_static_mutex_unlock (&mutex);
+	G_UNLOCK (mutex);
 
 	if (ret) {
 		return TRUE;
