@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -27,8 +27,8 @@
 
 #include <sys/statvfs.h>
 
-#include "xmms/xmms_log.h"
-#include "xmmspriv/xmms_statfs.h"
+#include <xmms/xmms_log.h>
+#include <xmmspriv/xmms_statfs.h>
 
 /**
  * This function uses the statfs() call to
