@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -20,7 +20,7 @@
  */
 
 
-#include "xmmspriv/xmms_symlink.h"
+#include <xmmspriv/xmms_symlink.h>
 
 gboolean
 xmms_symlink_file (gchar *source, gchar *dest)
