@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -21,27 +21,25 @@
 
 #include <string.h>
 
-#include "xmmspriv/xmms_plugin.h"
-#include "xmmspriv/xmms_xform.h"
-#include "xmmspriv/xmms_streamtype.h"
-#include "xmmspriv/xmms_medialib.h"
-#include "xmmspriv/xmms_utils.h"
-#include "xmmspriv/xmms_xform_plugin.h"
-#include "xmms/xmms_ipc.h"
-#include "xmms/xmms_log.h"
-#include "xmms/xmms_object.h"
-
-struct xmms_xform_object_St {
-	xmms_object_t obj;
-};
+#include <xmmspriv/xmms_plugin.h>
+#include <xmmspriv/xmms_xform.h>
+#include <xmmspriv/xmms_streamtype.h>
+#include <xmmspriv/xmms_medialib.h>
+#include <xmmspriv/xmms_utils.h>
+#include <xmmspriv/xmms_xform_plugin.h>
+#include <xmms/xmms_ipc.h>
+#include <xmms/xmms_log.h>
+#include <xmms/xmms_object.h>
 
 struct xmms_xform_St {
 	xmms_object_t obj;
 	struct xmms_xform_St *prev;
 
-	const xmms_xform_plugin_t *plugin;
+	xmms_xform_plugin_t *plugin;
 	xmms_medialib_entry_t entry;
 
+	xmms_medialib_t *medialib;
+
 	gboolean inited;
 
 	void *priv;
@@ -65,7 +63,7 @@ struct xmms_xform_St {
 	GHashTable *privdata;
 	GQueue *hotspots;
 
-	GList *browse_list;
+	xmmsv_t *browse_list;
 	xmmsv_t *browse_dict;
 	gint browse_index;
 
@@ -96,11 +94,7 @@ static xmms_xform_t *xmms_xform_new_effect (xmms_xform_t* last,
                                             GList *goal_formats,
                                             const gchar *name);
 static void xmms_xform_destroy (xmms_object_t *object);
-static void effect_callbacks_init (void);
-
-static GList *xmms_xform_client_browse (xmms_xform_object_t *obj, const gchar *url, xmms_error_t *error);
-
-#include "xform_ipc.c"
+static xmms_stream_type_t *xmms_xform_get_out_stream_type (xmms_xform_t *xform);
 
 void
 xmms_xform_browse_add_entry_property_str (xmms_xform_t *xform,
@@ -175,7 +169,6 @@ void
 xmms_xform_browse_add_entry (xmms_xform_t *xform, const gchar *filename,
                              guint32 flags)
 {
-	xmmsv_t *val;
 	const gchar *url;
 	gchar *efile, *eurl, *t;
 	gint l, isdir;
@@ -206,8 +199,11 @@ xmms_xform_browse_add_entry (xmms_xform_t *xform, const gchar *filename,
 	xmms_xform_browse_add_entry_property_str (xform, "path", t);
 	xmms_xform_browse_add_entry_property_int (xform, "isdir", isdir);
 
-	val = xform->browse_dict;
-	xform->browse_list = g_list_prepend (xform->browse_list, val);
+	if (xform->browse_list == NULL) {
+		xform->browse_list = xmmsv_new_list ();
+	}
+	xmmsv_list_append (xform->browse_list, xform->browse_dict);
+	xmmsv_unref (xform->browse_dict);
 
 	g_free (t);
 	g_free (efile);
@@ -215,57 +211,55 @@ xmms_xform_browse_add_entry (xmms_xform_t *xform, const gchar *filename,
 }
 
 static gint
-xmms_browse_list_sortfunc (gconstpointer a, gconstpointer b)
+xmms_browse_list_sortfunc (xmmsv_t **a, xmmsv_t **b)
 {
-	int r1, r2;
-	xmmsv_t *val1, *val2, *tmp1, *tmp2;
+	xmmsv_t *val1, *val2;
 	const gchar *s1, *s2;
+	int r1, r2;
 
-	val1 = (xmmsv_t *) a;
-	val2 = (xmmsv_t *) b;
+	g_return_val_if_fail (xmmsv_is_type (*a, XMMSV_TYPE_DICT), 0);
+	g_return_val_if_fail (xmmsv_is_type (*b, XMMSV_TYPE_DICT), 0);
 
-	g_return_val_if_fail (xmmsv_get_type (val1) == XMMSV_TYPE_DICT, 0);
-	g_return_val_if_fail (xmmsv_get_type (val2) == XMMSV_TYPE_DICT, 0);
-
-	r1 = xmmsv_dict_get (val1, "intsort", &tmp1);
-	r2 = xmmsv_dict_get (val2, "intsort", &tmp2);
+	r1 = xmmsv_dict_get (*a, "intsort", &val1);
+	r2 = xmmsv_dict_get (*b, "intsort", &val2);
 
 	if (r1 && r2) {
 		gint i1, i2;
 
-		if (!xmmsv_get_int (tmp1, &i1))
+		if (!xmmsv_get_int (val1, &i1))
 			return 0;
-		if (!xmmsv_get_int (tmp2, &i2))
+		if (!xmmsv_get_int (val2, &i2))
 			return 0;
 		return i1 > i2;
 	}
 
-	if (!xmmsv_dict_get (val1, "path", &tmp1))
+	if (!xmmsv_dict_get (*a, "path", &val1))
 		return 0;
-	if (!xmmsv_dict_get (val2, "path", &tmp2))
+	if (!xmmsv_dict_get (*b, "path", &val2))
 		return 0;
 
-	if (!xmmsv_get_string (tmp1, &s1))
+	if (!xmmsv_get_string (val1, &s1))
 		return 0;
-	if (!xmmsv_get_string (tmp2, &s2))
+	if (!xmmsv_get_string (val2, &s2))
 		return 0;
 
 	return xmms_natcmp (s1, s2);
 }
 
-GList *
+xmmsv_t *
 xmms_xform_browse_method (xmms_xform_t *xform, const gchar *url,
                           xmms_error_t *error)
 {
-	GList *list = NULL;
+	xmmsv_t *list = NULL;
 
 	if (xmms_xform_plugin_can_browse (xform->plugin)) {
+		xform->browse_list = xmmsv_new_list ();
 		if (!xmms_xform_plugin_browse (xform->plugin, xform, url, error)) {
 			return NULL;
 		}
 		list = xform->browse_list;
 		xform->browse_list = NULL;
-		list = g_list_sort (list, xmms_browse_list_sortfunc);
+		xmmsv_list_sort (list, xmms_browse_list_sortfunc);
 	} else {
 		xmms_error_set (error, XMMS_ERROR_GENERIC, "Couldn't handle that URL");
 	}
@@ -273,15 +267,15 @@ xmms_xform_browse_method (xmms_xform_t *xform, const gchar *url,
 	return list;
 }
 
-GList *
+xmmsv_t *
 xmms_xform_browse (const gchar *url, xmms_error_t *error)
 {
-	GList *list = NULL;
+	xmmsv_t *list = NULL;
 	gchar *durl;
 	xmms_xform_t *xform = NULL;
 	xmms_xform_t *xform2 = NULL;
 
-	xform = xmms_xform_new (NULL, NULL, 0, NULL);
+	xform = xmms_xform_new (NULL, NULL, NULL, 0, NULL);
 
 	durl = g_strdup (url);
 	xmms_medialib_decode_url (durl);
@@ -314,33 +308,6 @@ xmms_xform_browse (const gchar *url, xmms_error_t *error)
 	return list;
 }
 
-static GList *
-xmms_xform_client_browse (xmms_xform_object_t *obj, const gchar *url,
-                          xmms_error_t *error)
-{
-	return xmms_xform_browse (url, error);
-}
-
-static void
-xmms_xform_object_destroy (xmms_object_t *obj)
-{
-	xmms_xform_unregister_ipc_commands ();
-}
-
-xmms_xform_object_t *
-xmms_xform_object_init (void)
-{
-	xmms_xform_object_t *obj;
-
-	obj = xmms_object_new (xmms_xform_object_t, xmms_xform_object_destroy);
-
-	xmms_xform_register_ipc_commands (XMMS_OBJECT (obj));
-
-	effect_callbacks_init ();
-
-	return obj;
-}
-
 static void
 xmms_xform_destroy (xmms_object_t *object)
 {
@@ -362,8 +329,13 @@ xmms_xform_destroy (xmms_object_t *object)
 
 	g_free (xform->buffer);
 
-	xmms_object_unref (xform->out_type);
-	xmms_object_unref (xform->plugin);
+	if (xform->out_type) {
+		xmms_object_unref (xform->out_type);
+	}
+
+	if (xform->plugin) {
+		xmms_object_unref (xform->plugin);
+	}
 
 	if (xform->prev) {
 		xmms_object_unref (xform->prev);
@@ -373,15 +345,16 @@ xmms_xform_destroy (xmms_object_t *object)
 
 xmms_xform_t *
 xmms_xform_new (xmms_xform_plugin_t *plugin, xmms_xform_t *prev,
-                xmms_medialib_entry_t entry, GList *goal_hints)
+                xmms_medialib_t *medialib, xmms_medialib_entry_t entry,
+                GList *goal_hints)
 {
 	xmms_xform_t *xform;
 
 	xform = xmms_object_new (xmms_xform_t, xmms_xform_destroy);
 
-	xmms_object_ref (plugin);
-	xform->plugin = plugin;
+	xform->plugin = plugin ? xmms_object_ref (plugin) : NULL;
 	xform->entry = entry;
+	xform->medialib = medialib;
 	xform->goal_hints = goal_hints;
 	xform->lr.bufend = &xform->lr.buf[0];
 
@@ -405,7 +378,7 @@ xmms_xform_new (xmms_xform_plugin_t *plugin, xmms_xform_t *prev,
 			return NULL;
 		}
 		xform->inited = TRUE;
-		g_return_val_if_fail (xform->out_type, NULL);
+		g_return_val_if_fail (xmms_xform_get_out_stream_type (xform), NULL);
 	}
 
 	xform->buffer = g_malloc (READ_CHUNK);
@@ -444,22 +417,21 @@ xmms_xform_outdata_type_add (xmms_xform_t *xform, ...)
 void
 xmms_xform_outdata_type_set (xmms_xform_t *xform, xmms_stream_type_t *type)
 {
-	xmms_object_ref (type);
-	xform->out_type = type;
+	g_return_if_fail (xform->out_type == NULL);
+	xform->out_type = xmms_object_ref (type);
 }
 
 void
 xmms_xform_outdata_type_copy (xmms_xform_t *xform)
 {
-	xmms_object_ref (xform->prev->out_type);
-	xform->out_type = xform->prev->out_type;
+	xmms_stream_type_t *st = xmms_xform_get_out_stream_type (xform->prev);
+	xmms_xform_outdata_type_set (xform, st);
 }
 
 const char *
 xmms_xform_indata_find_str (xmms_xform_t *xform, xmms_stream_type_key_t key)
 {
-	const gchar *r;
-	r = xmms_stream_type_get_str (xform->prev->out_type, key);
+	const gchar *r = xmms_xform_indata_get_str (xform, key);
 	if (r) {
 		return r;
 	} else if (xform->prev) {
@@ -471,13 +443,13 @@ xmms_xform_indata_find_str (xmms_xform_t *xform, xmms_stream_type_key_t key)
 const char *
 xmms_xform_indata_get_str (xmms_xform_t *xform, xmms_stream_type_key_t key)
 {
-	return xmms_stream_type_get_str (xform->prev->out_type, key);
+	return xmms_xform_outtype_get_str (xform->prev, key);
 }
 
 gint
 xmms_xform_indata_get_int (xmms_xform_t *xform, xmms_stream_type_key_t key)
 {
-	return xmms_stream_type_get_int (xform->prev->out_type, key);
+	return xmms_xform_outtype_get_int (xform->prev, key);
 }
 
 xmms_stream_type_t *
@@ -492,31 +464,36 @@ xmms_xform_intype_get (xmms_xform_t *xform)
 	return xmms_xform_outtype_get (xform->prev);
 }
 
-
-
 const char *
 xmms_xform_outtype_get_str (xmms_xform_t *xform, xmms_stream_type_key_t key)
 {
-	return xmms_stream_type_get_str (xform->out_type, key);
+	const xmms_stream_type_t *st = xmms_xform_get_out_stream_type (xform);
+	return xmms_stream_type_get_str (st, key);
 }
 
 gint
 xmms_xform_outtype_get_int (xmms_xform_t *xform, xmms_stream_type_key_t key)
 {
-	return xmms_stream_type_get_int (xform->out_type, key);
+	const xmms_stream_type_t *st = xmms_xform_get_out_stream_type (xform);
+	return xmms_stream_type_get_int (st, key);
 }
 
+gboolean
+xmms_xform_metadata_mapper_match (xmms_xform_t *xform, const gchar *key, const gchar *value, gsize length)
+{
+	return xmms_xform_plugin_metadata_mapper_match (xform->plugin, xform, key, value, length);
+}
 
-void
+gboolean
 xmms_xform_metadata_set_int (xmms_xform_t *xform, const char *key, int val)
 {
-	XMMS_DBG ("Setting '%s' to %d", key, val);
 	g_hash_table_insert (xform->metadata, g_strdup (key),
 	                     xmmsv_new_int (val));
 	xform->metadata_changed = TRUE;
+	return TRUE;
 }
 
-void
+gboolean
 xmms_xform_metadata_set_str (xmms_xform_t *xform, const char *key,
                              const char *val)
 {
@@ -524,12 +501,12 @@ xmms_xform_metadata_set_str (xmms_xform_t *xform, const char *key,
 
 	if (!g_utf8_validate (val, -1, NULL)) {
 		xmms_log_error ("xform '%s' tried to set property '%s' to a NON UTF-8 string!", xmms_xform_shortname (xform), key);
-		return;
+		return FALSE;
 	}
 
 	if (xmms_xform_metadata_get_str (xform, key, &old)) {
 		if (strcmp (old, val) == 0) {
-			return;
+			return TRUE;
 		}
 	}
 
@@ -537,6 +514,8 @@ xmms_xform_metadata_set_str (xmms_xform_t *xform, const char *key,
 	                     xmmsv_new_string (val));
 
 	xform->metadata_changed = TRUE;
+
+	return TRUE;
 }
 
 static const xmmsv_t *
@@ -593,9 +572,9 @@ xmms_xform_metadata_get_str (xmms_xform_t *xform, const char *key,
 }
 
 typedef struct {
-	xmms_medialib_session_t *session;
 	xmms_medialib_entry_t entry;
-	guint32 source;
+	xmms_medialib_session_t *session;
+	gchar *source;
 } metadata_festate_t;
 
 static void
@@ -636,8 +615,9 @@ xmms_xform_metadata_collect_one (xmms_xform_t *xform, metadata_festate_t *info)
 	g_snprintf (src, sizeof (src), "plugin/%s",
 	            xmms_xform_shortname (xform));
 
-	info->source = xmms_medialib_source_to_id (info->session, src);
+	info->source = src;
 	g_hash_table_foreach (xform->metadata, add_metadatum, info);
+	info->source = NULL;
 
 	xform->metadata_changed = FALSE;
 }
@@ -665,7 +645,9 @@ xmms_xform_metadata_collect_r (xmms_xform_t *xform, metadata_festate_t *info,
 }
 
 static void
-xmms_xform_metadata_collect (xmms_xform_t *start, GString *namestr, gboolean rehashing)
+xmms_xform_metadata_collect (xmms_medialib_session_t *session,
+                             xmms_xform_t *start, GString *namestr,
+                             gboolean rehashing)
 {
 	metadata_festate_t info;
 	gint times_played;
@@ -673,10 +655,9 @@ xmms_xform_metadata_collect (xmms_xform_t *start, GString *namestr, gboolean reh
 	GTimeVal now;
 
 	info.entry = start->entry;
-	info.session = xmms_medialib_begin_write ();
 
-	times_played = xmms_medialib_entry_property_get_int (info.session,
-	                                                     info.entry,
+	info.session = session;
+	times_played = xmms_medialib_entry_property_get_int (session, info.entry,
 	                                                     XMMS_MEDIALIB_ENTRY_PROPERTY_TIMESPLAYED);
 
 	/* times_played == -1 if we haven't played this entry yet. so after initial
@@ -686,49 +667,49 @@ xmms_xform_metadata_collect (xmms_xform_t *start, GString *namestr, gboolean reh
 		times_played = 0;
 	}
 
-	last_started = xmms_medialib_entry_property_get_int (info.session,
-	                                                     info.entry,
+	last_started = xmms_medialib_entry_property_get_int (session, info.entry,
 	                                                     XMMS_MEDIALIB_ENTRY_PROPERTY_LASTSTARTED);
 
-	xmms_medialib_entry_cleanup (info.session, info.entry);
+	xmms_medialib_entry_cleanup (session, info.entry);
 
 	xmms_xform_metadata_collect_r (start, &info, namestr);
 
-	xmms_medialib_entry_property_set_str (info.session, info.entry,
+	xmms_medialib_entry_property_set_str (session, info.entry,
 	                                      XMMS_MEDIALIB_ENTRY_PROPERTY_CHAIN,
 	                                      namestr->str);
 
-	xmms_medialib_entry_property_set_int (info.session, info.entry,
+	xmms_medialib_entry_property_set_int (session, info.entry,
 	                                      XMMS_MEDIALIB_ENTRY_PROPERTY_TIMESPLAYED,
 	                                      times_played + (rehashing ? 0 : 1));
 
 	if (!rehashing || (rehashing && last_started)) {
 		g_get_current_time (&now);
 
-		xmms_medialib_entry_property_set_int (info.session, info.entry,
+		xmms_medialib_entry_property_set_int (session, info.entry,
 		                                      XMMS_MEDIALIB_ENTRY_PROPERTY_LASTSTARTED,
 		                                      (rehashing ? last_started : now.tv_sec));
 	}
 
-	xmms_medialib_entry_status_set (info.session, info.entry,
+	xmms_medialib_entry_status_set (session, info.entry,
 	                                XMMS_MEDIALIB_ENTRY_STATUS_OK);
-
-	xmms_medialib_end (info.session);
-	xmms_medialib_entry_send_update (info.entry);
 }
 
 static void
 xmms_xform_metadata_update (xmms_xform_t *xform)
 {
+	xmms_medialib_session_t *session;
 	metadata_festate_t info;
 
-	info.entry = xform->entry;
-	info.session = xmms_medialib_begin_write ();
+	g_return_if_fail (xform->medialib);
 
-	xmms_xform_metadata_collect_one (xform, &info);
+	do {
+		session = xmms_medialib_session_begin (xform->medialib);
 
-	xmms_medialib_end (info.session);
-	xmms_medialib_entry_send_update (info.entry);
+		info.entry = xform->entry;
+		info.session = session;
+
+		xmms_xform_metadata_collect_one (xform, &info);
+	} while (!xmms_medialib_session_commit (session));
 }
 
 static void
@@ -752,7 +733,7 @@ xmms_xform_auxdata_barrier (xmms_xform_t *xform)
 }
 
 void
-xmms_xform_auxdata_set_int (xmms_xform_t *xform, const char *key, int intval)
+xmms_xform_auxdata_set_int (xmms_xform_t *xform, const char *key, gint64 intval)
 {
 	xmmsv_t *val = xmmsv_new_int (intval);
 	xmms_xform_auxdata_set_val (xform, g_strdup (key), val);
@@ -817,34 +798,20 @@ xmms_xform_auxdata_has_val (xmms_xform_t *xform, const gchar *key)
 	return !!xmms_xform_auxdata_get_val (xform, key);
 }
 
-gboolean
-xmms_xform_auxdata_get_int (xmms_xform_t *xform, const gchar *key, gint32 *val)
-{
-	const xmmsv_t *obj;
-
-	obj = xmms_xform_auxdata_get_val (xform, key);
-	if (obj && xmmsv_get_type (obj) == XMMSV_TYPE_INT32) {
-		xmmsv_get_int (obj, val);
-		return TRUE;
+/* macro-magically define auxdata extractors */
+#define GEN_AUXDATA_EXTRACTOR_FUNC(typename, xmmsvtypename, type) \
+	gboolean \
+	xmms_xform_auxdata_get_##typename (xmms_xform_t *xform, const gchar *key, \
+	                                   type *val) \
+	{ \
+		const xmmsv_t *obj; \
+		obj = xmms_xform_auxdata_get_val (xform, key); \
+		return obj && xmmsv_get_##xmmsvtypename (obj, val); \
 	}
 
-	return FALSE;
-}
-
-gboolean
-xmms_xform_auxdata_get_str (xmms_xform_t *xform, const gchar *key,
-                            const gchar **val)
-{
-	const xmmsv_t *obj;
-
-	obj = xmms_xform_auxdata_get_val (xform, key);
-	if (obj && xmmsv_get_type (obj) == XMMSV_TYPE_STRING) {
-		xmmsv_get_string (obj, val);
-		return TRUE;
-	}
-
-	return FALSE;
-}
+GEN_AUXDATA_EXTRACTOR_FUNC (int32, int32, gint32);
+GEN_AUXDATA_EXTRACTOR_FUNC (int64, int64, gint64);
+GEN_AUXDATA_EXTRACTOR_FUNC (str, string, const gchar *);
 
 gboolean
 xmms_xform_auxdata_get_bin (xmms_xform_t *xform, const gchar *key,
@@ -854,7 +821,7 @@ xmms_xform_auxdata_get_bin (xmms_xform_t *xform, const gchar *key,
 
 	obj = xmms_xform_auxdata_get_val (xform, key);
 	if (obj && xmmsv_get_type (obj) == XMMSV_TYPE_BIN) {
-		xmmsv_get_bin (obj, data, datalen);
+		xmmsv_get_bin (obj, data, (guint *) datalen);
 		return TRUE;
 	}
 
@@ -1006,7 +973,7 @@ xmms_xform_this_read (xmms_xform_t *xform, gpointer buf, gint siz,
 					                           xform->buffersize);
 				}
 
-				g_memmove (xform->buffer + xform->buffered, buf + read, res);
+				memmove (xform->buffer + xform->buffered, buf + read, res);
 				xform->buffered += res;
 				break;
 			}
@@ -1142,7 +1109,7 @@ xmms_xform_get_url (xmms_xform_t *xform)
 
 typedef struct match_state_St {
 	xmms_xform_plugin_t *match;
-	xmms_stream_type_t *out_type;
+	const xmms_stream_type_t *out_type;
 	gint priority;
 } match_state_t;
 
@@ -1186,14 +1153,14 @@ xmms_xform_find (xmms_xform_t *prev, xmms_medialib_entry_t entry,
 	match_state_t state;
 	xmms_xform_t *xform = NULL;
 
-	state.out_type = prev->out_type;
+	state.out_type = xmms_xform_get_out_stream_type (prev);
 	state.match = NULL;
 	state.priority = -1;
 
 	xmms_plugin_foreach (XMMS_PLUGIN_TYPE_XFORM, xmms_xform_match, &state);
 
 	if (state.match) {
-		xform = xmms_xform_new (state.match, prev, entry, goal_hints);
+		xform = xmms_xform_new (state.match, prev, prev->medialib, entry, goal_hints);
 	} else {
 		XMMS_DBG ("Found no matching plugin...");
 	}
@@ -1213,9 +1180,11 @@ xmms_xform_iseos (xmms_xform_t *xform)
 	return ret;
 }
 
-const xmms_stream_type_t *
+static xmms_stream_type_t *
 xmms_xform_get_out_stream_type (xmms_xform_t *xform)
 {
+	if (!xform->out_type)
+		return xmms_xform_plugin_get_out_stream_type (xform->plugin);
 	return xform->out_type;
 }
 
@@ -1256,9 +1225,9 @@ outdata_type_metadata_collect (xmms_xform_t *xform)
 {
 	gint val;
 	const char *mime;
-	xmms_stream_type_t *type;
+	const xmms_stream_type_t *type;
 
-	type = xform->out_type;
+	type = xmms_xform_get_out_stream_type (xform);
 	mime = xmms_stream_type_get_str (type, XMMS_STREAM_TYPE_MIMETYPE);
 	if (strcmp (mime, "audio/pcm") != 0) {
 		return;
@@ -1288,7 +1257,8 @@ outdata_type_metadata_collect (xmms_xform_t *xform)
 }
 
 static xmms_xform_t *
-chain_setup (xmms_medialib_entry_t entry, const gchar *url, GList *goal_formats)
+chain_setup (xmms_medialib_t *medialib, xmms_medialib_entry_t entry,
+             const gchar *url, GList *goal_formats)
 {
 	xmms_xform_t *xform, *last;
 	gchar *durl, *args;
@@ -1297,7 +1267,7 @@ chain_setup (xmms_medialib_entry_t entry, const gchar *url, GList *goal_formats)
 		entry = 1; /* FIXME: this is soooo ugly, don't do this */
 	}
 
-	xform = xmms_xform_new (NULL, NULL, 0, goal_formats);
+	xform = xmms_xform_new (NULL, NULL, medialib , 0, goal_formats);
 
 	durl = g_strdup (url);
 
@@ -1330,7 +1300,6 @@ chain_setup (xmms_medialib_entry_t entry, const gchar *url, GList *goal_formats)
 	                             "application/x-url", XMMS_STREAM_TYPE_URL,
 	                             durl, XMMS_STREAM_TYPE_END);
 
-	g_free (durl);
 
 	last = xform;
 
@@ -1338,8 +1307,9 @@ chain_setup (xmms_medialib_entry_t entry, const gchar *url, GList *goal_formats)
 		xform = xmms_xform_find (last, entry, goal_formats);
 		if (!xform) {
 			xmms_log_error ("Couldn't set up chain for '%s' (%d)",
-			                url, entry);
+			                durl, entry);
 			xmms_object_unref (last);
+			g_free (durl);
 
 			return NULL;
 		}
@@ -1347,35 +1317,39 @@ chain_setup (xmms_medialib_entry_t entry, const gchar *url, GList *goal_formats)
 		last = xform;
 	} while (!has_goalformat (xform, goal_formats));
 
+	g_free (durl);
+
 	outdata_type_metadata_collect (last);
 
 	return last;
 }
 
 static void
-chain_finalize (xmms_xform_t *xform, xmms_medialib_entry_t entry,
+chain_finalize (xmms_medialib_session_t *session,
+                xmms_xform_t *xform, xmms_medialib_entry_t entry,
                 const gchar *url, gboolean rehashing)
 {
 	GString *namestr;
+	gchar *durl;
+
+	durl = g_strdup (url);
+	xmms_medialib_decode_url (durl);
 
 	namestr = g_string_new ("");
-	xmms_xform_metadata_collect (xform, namestr, rehashing);
+	xmms_xform_metadata_collect (session, xform, namestr, rehashing);
 	xmms_log_info ("Successfully setup chain for '%s' (%d) containing %s",
-	               url, entry, namestr->str);
+	               durl, entry, namestr->str);
 
 	g_string_free (namestr, TRUE);
+	g_free (durl);
 }
 
 static gchar *
-get_url_for_entry (xmms_medialib_entry_t entry)
+get_url_for_entry (xmms_medialib_session_t *session, xmms_medialib_entry_t entry)
 {
-	xmms_medialib_session_t *session;
 	gchar *url = NULL;
 
-	session = xmms_medialib_begin ();
-	url = xmms_medialib_entry_property_get_str (session, entry,
-	                                            XMMS_MEDIALIB_ENTRY_PROPERTY_URL);
-	xmms_medialib_end (session);
+	url = xmms_medialib_entry_property_get_str (session, entry, XMMS_MEDIALIB_ENTRY_PROPERTY_URL);
 
 	if (!url) {
 		xmms_log_error ("Couldn't get url for entry (%d)", entry);
@@ -1385,25 +1359,47 @@ get_url_for_entry (xmms_medialib_entry_t entry)
 }
 
 xmms_xform_t *
-xmms_xform_chain_setup (xmms_medialib_entry_t entry, GList *goal_formats,
-                        gboolean rehash)
+xmms_xform_chain_setup (xmms_medialib_t *medialib, xmms_medialib_entry_t entry,
+                        GList *goal_formats, gboolean rehash)
+{
+	xmms_medialib_session_t *session;
+	xmms_xform_t *ret = NULL;
+
+	do {
+		session = xmms_medialib_session_begin (medialib);
+		if (ret != NULL)
+			xmms_object_unref (ret);
+		ret = xmms_xform_chain_setup_session (medialib, session, entry, goal_formats, rehash);
+	} while (!xmms_medialib_session_commit (session));
+
+	return ret;
+}
+
+xmms_xform_t *
+xmms_xform_chain_setup_session (xmms_medialib_t *medialib,
+                                xmms_medialib_session_t *session,
+                                xmms_medialib_entry_t entry,
+                                GList *goal_formats, gboolean rehash)
 {
 	gchar *url;
 	xmms_xform_t *xform;
 
-	if (!(url = get_url_for_entry (entry))) {
+	if (!(url = get_url_for_entry (session, entry))) {
 		return NULL;
 	}
 
-	xform = xmms_xform_chain_setup_url (entry, url, goal_formats, rehash);
+	xform = xmms_xform_chain_setup_url_session (medialib, session, entry,
+	                                            url, goal_formats, rehash);
 	g_free (url);
 
 	return xform;
 }
 
 xmms_xform_t *
-xmms_xform_chain_setup_url (xmms_medialib_entry_t entry, const gchar *url,
-                            GList *goal_formats, gboolean rehash)
+xmms_xform_chain_setup_url_session (xmms_medialib_t *medialib,
+                                    xmms_medialib_session_t *session,
+                                    xmms_medialib_entry_t entry, const gchar *url,
+                                    GList *goal_formats, gboolean rehash)
 {
 	xmms_xform_t *last;
 	xmms_plugin_t *plugin;
@@ -1411,7 +1407,7 @@ xmms_xform_chain_setup_url (xmms_medialib_entry_t entry, const gchar *url,
 	gboolean add_segment = FALSE;
 	gint priority;
 
-	last = chain_setup (entry, url, goal_formats);
+	last = chain_setup (medialib, entry, url, goal_formats);
 	if (!last) {
 		return NULL;
 	}
@@ -1423,8 +1419,8 @@ xmms_xform_chain_setup_url (xmms_medialib_entry_t entry, const gchar *url,
 	/* if segment plugin input is the same as current output, include it
 	 * for collecting additional duration metadata on audio entries */
 	if (xform_plugin) {
-		add_segment = xmms_xform_plugin_supports (xform_plugin,
-		                                          last->out_type,
+		const xmms_stream_type_t *st = xmms_xform_get_out_stream_type (last);
+		add_segment = xmms_xform_plugin_supports (xform_plugin, st,
 		                                          &priority);
 		xmms_object_unref (plugin);
 	}
@@ -1445,10 +1441,29 @@ xmms_xform_chain_setup_url (xmms_medialib_entry_t entry, const gchar *url,
 		}
 	}
 
-	chain_finalize (last, entry, url, rehash);
+	chain_finalize (session, last, entry, url, rehash);
 	return last;
 }
 
+xmms_xform_t *
+xmms_xform_chain_setup_url (xmms_medialib_t *medialib,
+                            xmms_medialib_entry_t entry, const gchar *url,
+                            GList *goal_formats, gboolean rehash)
+{
+	xmms_medialib_session_t *session;
+	xmms_xform_t *ret = NULL;
+
+	do {
+		session = xmms_medialib_session_begin (medialib);
+		if (ret != NULL)
+			xmms_object_unref (ret);
+		ret = xmms_xform_chain_setup_url_session (medialib, session, entry, url,
+		                                          goal_formats, rehash);
+	} while (!xmms_medialib_session_commit (session));
+
+	return ret;
+}
+
 xmms_config_property_t *
 xmms_xform_config_lookup (xmms_xform_t *xform, const gchar *path)
 {
@@ -1510,7 +1525,7 @@ xmms_xform_new_effect (xmms_xform_t *last, xmms_medialib_entry_t entry,
 		return last;
 	}
 
-	xform = xmms_xform_new (xform_plugin, last, entry, goal_formats);
+	xform = xmms_xform_new (xform_plugin, last, last->medialib, entry, goal_formats);
 
 	if (xform) {
 		xmms_object_unref (last);
@@ -1525,87 +1540,3 @@ xmms_xform_new_effect (xmms_xform_t *last, xmms_medialib_entry_t entry,
 	xmms_object_unref (plugin);
 	return last;
 }
-
-static void
-update_effect_properties (xmms_object_t *object, xmmsv_t *data,
-                          gpointer userdata)
-{
-	gint effect_no = GPOINTER_TO_INT (userdata);
-	const gchar *name;
-
-	xmms_config_property_t *cfg;
-	xmms_xform_plugin_t *xform_plugin;
-	xmms_plugin_t *plugin;
-	gchar key[64];
-
-	name = xmms_config_property_get_string ((xmms_config_property_t *) object);
-
-	if (name[0]) {
-		plugin = xmms_plugin_find (XMMS_PLUGIN_TYPE_XFORM, name);
-		if (!plugin) {
-			xmms_log_error ("Couldn't find any effect named '%s'", name);
-		} else {
-			xform_plugin = (xmms_xform_plugin_t *) plugin;
-			xmms_xform_plugin_config_property_register (xform_plugin, "enabled",
-			                                            "1", NULL, NULL);
-			xmms_object_unref (plugin);
-		}
-
-		/* setup new effect.order.n */
-		g_snprintf (key, sizeof (key), "effect.order.%i", effect_no + 1);
-
-		cfg = xmms_config_lookup (key);
-		if (!cfg) {
-			xmms_config_property_register (key, "", update_effect_properties,
-			                               GINT_TO_POINTER (effect_no + 1));
-		}
-	}
-}
-
-static void
-effect_callbacks_init (void)
-{
-	gint effect_no;
-
-	xmms_config_property_t *cfg;
-	xmms_xform_plugin_t *xform_plugin;
-	xmms_plugin_t *plugin;
-	gchar key[64];
-	const gchar *name;
-
-	for (effect_no = 0; ; effect_no++) {
-		g_snprintf (key, sizeof (key), "effect.order.%i", effect_no);
-
-		cfg = xmms_config_lookup (key);
-		if (!cfg) {
-			break;
-		}
-		xmms_config_property_callback_set (cfg, update_effect_properties,
-		                                   GINT_TO_POINTER (effect_no));
-
-		name = xmms_config_property_get_string (cfg);
-		if (!name[0]) {
-			continue;
-		}
-
-		plugin = xmms_plugin_find (XMMS_PLUGIN_TYPE_XFORM, name);
-		if (!plugin) {
-			xmms_log_error ("Couldn't find any effect named '%s'", name);
-			continue;
-		}
-
-		xform_plugin = (xmms_xform_plugin_t *) plugin;
-		xmms_xform_plugin_config_property_register (xform_plugin, "enabled",
-		                                            "1", NULL, NULL);
-
-		xmms_object_unref (plugin);
-	}
-
-	/* the name stored in the last present property was not "" or there was no
-	   last present property */
-	if ((!effect_no) || name[0]) {
-			xmms_config_property_register (key, "", update_effect_properties,
-			                               GINT_TO_POINTER (effect_no));
-	}
-}
-
