@@ -1,11 +1,9 @@
-import Task
-import TaskGen
-from TaskGen import extension
+from waflib import TaskGen
 import sys
 
 TaskGen.declare_chain(
 		name='genpy',
-		rule='${PYTHON} ${SRC} > ${TGT}',
+		rule='${PYTHON} ${SRC} ${GENPY_ARGS} > ${TGT}',
 		reentrant=True,
 		color='BLUE',
 		ext_in='.genpy',
@@ -15,4 +13,5 @@ TaskGen.declare_chain(
 def configure(conf):
     conf.env['PYTHON'] = sys.executable
     conf.env['GENPY_EXT'] = ['.genpy']
+    conf.env['GENPY_ARGS'] = ''
     return True
